! All subroutines common to both programs except those that are part of 
! the Tsyganenko model have been collected in this file.
!
! HG 2010-08-28
!
!------------------------------------------------------------------------

subroutine InitialiseParticles(vel,pos,X,Y,Z,vx0,vy0,vz0,vth,Lside,Npart)

! This subroutine initialises a particle distribution in a cube the 
! side of which is Lside, and which is aligned with the axes of 
! the cartesian coordinate system.


! No implicit variables
  implicit none

  integer Npart, pp

  real X, Y, Z, vx0, vy0, vz0

  real vel(Npart,3),pos(Npart,3), vth,Lside,rndp(3)



  do pp = 1, Npart
     call random_number(rndp)
     pos(pp,1) = X + rndp(1)*Lside - 0.5*Lside      
     pos(pp,2) = Y + rndp(2)*Lside - 0.5*Lside      
     pos(pp,3) = Z + rndp(3)*Lside - 0.5*Lside      
  end do



! Use the subroutine Gaussian to produce thermal
! distributions.
! Maxwellian f(vx) ~ exp(-vx^2/vth^2) etc. => vth=sqrt(2*kB*T/m)

  call  Gaussian(vel,Npart,1,vx0,vth/sqrt(2.0))

  call  Gaussian(vel,Npart,2,vy0,vth/sqrt(2.0))

  call  Gaussian(vel,Npart,3,vz0,vth/sqrt(2.0))

  return
end subroutine InitialiseParticles

!------------------------------------------------------------------------

subroutine Gaussian(x,N,comp,mu,sigma)

! Generates a normally distributed sequence of random numbers
! f(v) ~ exp(-(v-mu)^2/(2*sigma^2)


! No implicit variables
  implicit none

! Parameters
  integer N, comp
  real x(N,3), mu, sigma


! Local variables
  integer ii
  real pi,rndp(2)


  pi = 3.14159265358979323846264338327950288419716d0



  do ii = 1, N
     call random_number(rndp)
     x(ii,comp) = sqrt(-2.0*log(rndp(1)))* &
          cos(2.0*pi*rndp(2))*sigma+mu
  end do


  return 
end subroutine Gaussian

!------------------------------------------------------------------------

subroutine Boris(velocity_in,velocity_out,E,B,dt,mass,charge)

! This is the velocity update. It uses the Boris algorithm as described 
! by, e.g. Birdsall and Langdon.
!
!@Book{BL91,
!  author =	 {C. K. Birdsall and A. B. Langdon},
!  title =	 {Plasma physics via computer simulation},
!  publisher =	 "IOP Publishing Ltd",
!  address =	 "Dirac House, Temple Back, Bristol BS1 6BE, UK",
!  year =	 1991,
!  note =	 "{ISBN} 0 7503 1025 1"
!}
!

! No implicit variables
  implicit none



! Parameters
  real B(3)
  double precision velocity_in(3), velocity_out(3)
  double precision E(3)
  double precision dt, mass, charge


! Local variables
  double precision s(3), t(3), vplus(3), vmin(3), &
       vmid(3), eta, etadt2
  integer i


! Set constant local variables
  eta = charge/mass
  etadt2 = eta*dt/2.0d0

  
! Calculate t and vmin (first half acceleration)
  do i = 1, 3
     t(i) = etadt2*B(i)
     vmin(i) = velocity_in(i)+etadt2*E(i)
  end do

! Cross product
  vmid(1) = vmin(1) + vmin(2)*t(3)-vmin(3)*t(2)
  vmid(2) = vmin(2) + vmin(3)*t(1)-vmin(1)*t(3)
  vmid(3) = vmin(3) + vmin(1)*t(2)-vmin(2)*t(1)

! Calculate s
  do i = 1, 3
     s(i) = 2.0d0*t(i)/(1.0d0 + t(1)**2.0d0 + t(2)**2.0d0 + t(3)**2.0d0)
  end do

! Another cross product
  vplus(1) = vmin(1) + vmid(2)*s(3)-vmid(3)*s(2)
  vplus(2) = vmin(2) + vmid(3)*s(1)-vmid(1)*s(3)
  vplus(3) = vmin(3) + vmid(1)*s(2)-vmid(2)*s(1)

! Loop and set new velocities (second half acceleration)
  do i= 1, 3
     velocity_out(i) = vplus(i) + etadt2*E(i)
  end do


  return
end subroutine Boris

!------------------------------------------------------------------------

subroutine SpeedMomentAngle(vv,mu,pangle,velocity,Bin,mass)

! No implicit variables
  implicit none

  real Bin(3)

  double precision mass

  double precision velocity(3), vv, mu, pangle

! local variables
  double precision bb, bb2, vperp2, B(3)

  B=dble(Bin)                              ! convert to double precision
  bb2=B(1)**2.0d0+B(2)**2.0d0+B(3)**2.0d0  ! magnitude of B squared
  bb=sqrt(bb2)                             ! magnitude of B

  vv=sqrt(velocity(1)**2.0d0+velocity(2)**2.0d0+velocity(3)**2.0d0)
  vperp2=vv**2.0d0-(velocity(1)*B(1)+velocity(2)*B(2)+ &
       velocity(3)*B(3))**2.0d0/bb2
  mu=mass*vperp2/(2*bb)
  pangle=acos((velocity(1)*B(1)+velocity(2)*B(2)+velocity(3)*B(3))/(vv*bb))

  return
end subroutine SpeedMomentAngle

!------------------------------------------------------------------------

subroutine GetInput(iyear, iday, dhtime, Xgse, Ygse, Zgse, &
     vxgse, vygse, vzgse, Kp, Npart, Nsteps, vth, Lside, dt, mass, charge)

  implicit none

  integer iyear, iday, Kp, Npart, Nsteps
  real Xgse, Ygse, Zgse, vxgse, vygse, vzgse, dhtime, vth, Lside
  double precision dt, mass, charge

  character indata*132, filename*242
  integer i, j, k

  ! Defaults
  iyear  = 2000
  iday   = 1
  dhtime = 1.0
  Xgse   = 0.0
  Ygse   = 0.0
  Zgse   = 1.0e7
  vxgse  = 0.0
  vygse  = 0.0
  vzgse  = 1.0e5
  Kp     = 3
  Npart  = 1
  Nsteps = 10000
  vth    = 1.0e5
  Lside  = 1.0e6
  dt     = 0.1d0
  mass   = 2.67619d-26
  charge = 1.602176487d-19

  filename='inputham.m'
  open(unit=1,file=filename,status='old')

  read (1,'(a)') indata
  do while (index(indata,'%END')+index(indata,'%end') == 0)
     i=index(indata,'=')
     if (i>0) then
        do j=1,i-1
           if(indata(j:j) /= ' ') exit
        enddo
        ! If there is a semicolon on the line, ignore it and everything beyond.
        k = index(indata(i+1:132),';')
        if (k==0) then
           k = 133-i
        end if
        select case (indata(j:i-1))
        case ('iyear')
           read (indata(i+1:i+k-1),*) iyear
        case ('iday')
           read (indata(i+1:i+k-1),*) iday
        case ('dhtime')
           read (indata(i+1:i+k-1),*) dhtime
        case ('Xgse')
           read (indata(i+1:i+k-1),*) Xgse
        case ('Ygse')
           read (indata(i+1:i+k-1),*) Ygse
        case ('Zgse')
           read (indata(i+1:i+k-1),*) Zgse
        case ('vxgse')
           read (indata(i+1:i+k-1),*) vxgse
        case ('vygse')
           read (indata(i+1:i+k-1),*) vygse
        case ('vzgse')
           read (indata(i+1:i+k-1),*) vzgse
        case ('Kp')
           read (indata(i+1:i+k-1),*) Kp
        case ('Npart')
           read (indata(i+1:i+k-1),*) Npart
        case ('Nsteps')
           read (indata(i+1:i+k-1),*) Nsteps
        case ('vth')
           read (indata(i+1:i+k-1),*) vth
        case ('Lside')
           read (indata(i+1:i+k-1),*) Lside
        case ('dt')
           read (indata(i+1:i+k-1),*) dt
        case ('mass')
           read (indata(i+1:i+k-1),*) mass
        case ('charge')
           read (indata(i+1:i+k-1),*) charge
        case default
           if (index(indata(j:j+1),'%')==0) then
              write (*,*) 'Input quantity ', indata(j:i-1), ' is unknown.'
           end if
        end select
     end if
     read (1,'(a)') indata
  end do

  close(1)

end subroutine GetInput

!------------------------------------------------------------------------
