! This program computes trajectories and saves various quantities 
! for statistical purposes.

program hamlet

  implicit none

  integer iyear, iday, Kp, isec, iy, Npart, Nsteps, ii, jj, kk

  real parmod(10), Xgsm, Ygsm, Zgsm, Xgse, Ygse, Zgse, &
       vxgse, vygse, vzgse, &
       dhtime, hour, minute, Bgsm(3), Bgse(3), posgse(3), velgse(3)

  real Lside, vth

  double precision E(3), dt, mass, charge, rEarth, rr

  real, allocatable :: position_init_gse(:,:), position_init_gsm(:,:), &
       velocity_init_gse(:,:), velocity_init_gsm(:,:)

  double precision, allocatable :: position(:,:), velocity(:,:), &
       startvv(:), startmu(:), startpangle(:), &
       stopvv(:), stopmu(:), stoppangle(:)

  integer, allocatable :: fate(:)

  external T89C


! Initialise a couple of things
  parameter(rEarth = 6371.2d3)
  E=0.0d0 !We are assuming that there is no electric field
  ! The velocity integration in the Boris subroutine does, however, 
  ! allow us define an electric fieldin the E variable, should we 
  ! ever want to.


! Read input data
  call GetInput(iyear, iday, dhtime, Xgse, Ygse, Zgse, &
       vxgse, vygse, vzgse, Kp, Npart, Nsteps, vth, Lside, dt, mass, charge)
!!$  open(unit=1,file='inputhamlet.dat',status='old')
!!$  read(1,*) iyear, iday, dhtime, Xgse, Ygse, Zgse, vxgse, vygse, vzgse, &
!!$       Kp, Npart, Nsteps, &
!!$       vth, Lside, dt, mass, charge
!!$  close(1)

 
! Allocate variables for initial position and velocity, and for 
! particle position and velocity during flight.
  allocate(position_init_gse(Npart,3))
  allocate(position_init_gsm(Npart,3))
  allocate(velocity_init_gse(Npart,3))
  allocate(velocity_init_gsm(Npart,3))
  allocate(position(Nsteps,3))
  allocate(velocity(Nsteps,3))
  allocate(startvv(Npart))
  allocate(startmu(Npart))
  allocate(startpangle(Npart))
  allocate(stopvv(Npart))
  allocate(stopmu(Npart))
  allocate(stoppangle(Npart))
  allocate(fate(Npart))

! This is something Tsyganenko wants us to do before calling GSMGSE for 
! the first time (and subsequently every time the values of iyear, iday,
! ihour, min, and  isec have been changed).
  iy=iyear
  hour=aint(dhtime)
  minute=aint((dhtime-hour)*60.0);
  isec=int(((dhtime-hour)*60.0-minute)*60.0)
  call recalc(iyear,iday,int(hour),int(minute),isec)
 
! In T89C, at least, parmod is just a dummy variable, so I am setting 
! it to all zeros.
  parmod=0.0
! The variable fate indicates what happened to the particle. Initially 0.
  fate=0         ! fate = 0 : reached Nsteps iterations
                 ! fate = 1 : particle left through inner boundary
                 ! fate = 2 : particle left through outer boundary

! Give us particles! 
  call InitialiseParticles(velocity_init_gse,position_init_gse, &
       Xgse,Ygse,Zgse,vxgse,vygse,vzgse,vth,Lside,Npart)



! Convert our input GSE coordinates to GSM. The strategy is to perform 
! the calculations in GSM and to convert the results just before 
! saving them. 
  call gsmgse(Xgsm,Ygsm,Zgsm,Xgse,Ygse,Zgse,-1)
  do ii = 1, Npart
     call gsmgse(position_init_gsm(ii,1), &
          position_init_gsm(ii,2),position_init_gsm(ii,3), &
          position_init_gse(ii,1), &
          position_init_gse(ii,2),position_init_gse(ii,3),-1)
     call gsmgse(velocity_init_gsm(ii,1), &
          velocity_init_gsm(ii,2),velocity_init_gsm(ii,3), &
          velocity_init_gse(ii,1), &
          velocity_init_gse(ii,2),velocity_init_gse(ii,3),-1)
  end do


! Here starts the main loop that computes the trajectories
  do ii=1,Npart
! Convert the initial position and velocity to double precision
     position(1,:)=dble(position_init_gsm(ii,:))
     velocity(1,:)=dble(velocity_init_gsm(ii,:))

! Find initial B-field
     call bfield(real(position(1,1)/rEarth),real(position(1,2)/rEarth), &
          real(position(1,3)/rEarth),Bgsm(1),Bgsm(2),Bgsm(3),Kp,parmod,T89C)

! Take half a step back to be ready for leapfrog action
     call Boris(velocity(1,:),velocity(1,:),E,Bgsm*1e-9,-dt/2.0d0,mass,charge)
 
! calculate  vv, mu and pangle, for the first time step for saving later.
     call SpeedMomentAngle(startvv(ii),startmu(ii),startpangle(ii), &
          velocity(1,:),Bgsm,mass)

! Compute the trajectory
     do jj=1,Nsteps-1
! Find B-field
        call bfield(real(position(jj,1)/rEarth), &
             real(position(jj,2)/rEarth),real(position(jj,3)/rEarth), &
             Bgsm(1),Bgsm(2),Bgsm(3),Kp,parmod,T89C)

! Boris gives us the new velocity
        call Boris(velocity(jj,:),velocity(jj+1,:),E,Bgsm*1e-9,dt,mass,charge)

! Now, push!
        position(jj+1,:) = position(jj,:) + velocity(jj+1,:)*dt

! Check that we are still there
        rr=sqrt(position(jj+1,1)**2+position(jj+1,2)**2+position(jj+1,3)**2)
        if (rr < rEarth+85.0d3) then
           fate=1
           exit
        end if
        if (rr > 20.0d0*rEarth) then
           fate=2
           exit
        end if
     end do

! Compute vv, mu and pangle, i.e. the speed magnetic moment and 
! pitch angle, at the end of the particle trajectory. 
     call SpeedMomentAngle(stopvv(ii),stopmu(ii),stoppangle(ii), &
          velocity(jj+1,:),Bgsm,mass)

     if (ii/1000==ii/1000.0) then
!!$        write(*,fmt='(a, I7.5, a, I7.5)') 'particle ', ii, '/', Npart
        write(*,*) 'particle ', ii, ' /', Npart
     end if
  end do

  call DumpStartAndStop(startvv,startmu,startpangle, &
       stopvv,stopmu,stoppangle,fate,Npart)

end program hamlet

!------------------------------------------------------------------------

subroutine DumpStartAndStop(startvv,startmu,startpangle, &
       stopvv,stopmu,stoppangle,fate,Npart)

  integer Npart

  integer fate(Npart)

  double precision startvv(Npart), startmu(Npart), startpangle(Npart), &
       stopvv(Npart), stopmu(Npart), stoppangle(Npart)


  open(unit=2,file='StartAndStop.dat',status='new')

  do ii=1,Npart
     write (2,fmt='(6E14.6, i3)') &
          startvv(ii), startmu(ii), startpangle(ii), &
          stopvv(ii), stopmu(ii), stoppangle(ii), fate(ii)
  end do

  close(2)

end subroutine DumpStartAndStop

!------------------------------------------------------------------------
